#!/usr/bin/env node
import { promisify } from "util";
import cp from "child_process";
import path from "path";
import fs from "fs";
import { TextColor } from "ink";

// convert libs to promises
const exec = promisify(cp.exec);
const rm = promisify(fs.rm);

if (process.argv.length < 3) {
  console.log("You have to provide a name to this new project.");
  console.log("For example:");
  console.log("    npx omahti-cms-web-starter new-project");
  process.exit(1);
}

const projectName = process.argv[2];
const currentPath = process.cwd();
const projectPath = path.join(currentPath, projectName);

const git_repo = "https://github.com/OmahTI-UGM/omahti-cms-web-starter.git";

// create project directory
if (fs.existsSync(projectPath)) {
  console.log(
    `The file ${projectName} already exist in the current directory, please give it another name.`,
  );
  process.exit(1);
} else {
  fs.mkdirSync(projectPath);
}

try {
  // Use ink for informational messages
  const spinner = new TextColor("cyan");

  spinner.print("Downloading files...");

  // clone the repo into the project folder -> creates the new boilerplate
  await exec(`git clone ${git_repo} ${projectName} --quiet`);

  spinner.print("Removing useless files");

  // remove my git history
  const rmGit = rm(path.join(projectPath, ".git"), {
    recursive: true,
    force: true,
  });
  // remove the installation file
  const rmBin = rm(path.join(projectPath, "bin"), {
    recursive: true,
    force: true,
  });
  await Promise.all([rmGit, rmBin]);

  process.chdir(projectPath);
  // remove the packages needed for cli
  await exec("npm uninstall ora cli-spinners");

  spinner.print("Installing dependencies...");
  await exec("npm install");

  spinner.print("The installation is done!");
  console.log("You can now run your app with:");
  console.log(`    cd ${projectName}`);
  console.log(`    npm run dev`);
} catch (error) {
  // clean up in case of error, so the user does not have to do it manually
  fs.rmSync(projectPath, { recursive: true, force: true });
  console.log(error);
}
